#!/usr/bin/python2

import time
import os
import traceback

from os.path import join


from process.sequence import AlignmentList
import process.data as data


data_dir = "/home/diogo/Diogo/Science/Scripts/packages/TriFusion_benchmarks/data/"
db_path = "/home/diogo/Diogo/TriFusion_testdata/benchmark.db"
res_path = "/home/diogo/Diogo/TriFusion_testdata/results.csv"
bench_dir = "/home/diogo/Diogo/TriFusion_testdata/benchmarks_out"
res = []

ds1 = join(data_dir, "614_protein")
ds2 = join(data_dir, "3093_protein")
ds3 = join(data_dir, "3093_141_protein")
ds4 = join(data_dir, "3093_376_protein")
ds5 = join(data_dir, "7378_dna")
ds6 = join(data_dir, "1_40_large")
ds7 = join(data_dir, "1_40_large_interleave")
ds8 = join(data_dir, "1_40_large_interleave_nex")
ds9 = join(data_dir, "1_376_large")
ds10 = join(data_dir, "2_big_data")

ds_list = [[join(x, j) for j in os.listdir(x)] for x in [ds1, ds2, ds3, ds4,
                                                         ds5, ds6, ds7, ds8,
                                                         ds9, ds10]]

# rev_ds1 = join(data_dir, "614_protein_rev")
# rev_ds2 = join(data_dir, "3093_protein_rev")
# rev_ds3 = join(data_dir, "3093_141_protein_rev")
# rev_ds4 = join(data_dir, "3093_376_protein_rev")
# rev_ds5 = join(data_dir, "7378_dna_rev")
# rev_ds8 = join(data_dir, "2_big_data_rev")
#
# rev_ds_list = [[join(x, j) for j in os.listdir(x) if j.endswith(".phy")] for
#                x in [rev_ds1, rev_ds2, rev_ds3, rev_ds4, rev_ds5, rev_ds8]]
# part_files = [[join(x, j) for j in os.listdir(x) if not j.endswith(".phy")] for
#                x in [rev_ds1, rev_ds2, rev_ds3, rev_ds4, rev_ds5, rev_ds8]]

# Filter data
flist = [["Postia_placenta"], ["Postia_placenta"], ["Postia_placenta"],
         ["Postia_placenta"], ["535_12"], ["Ang_30"], ["Ang_30"], ["Ang_30"]]

class CleanUp(object):

    def __init__(self, func):

        self.func = func

    def __call__(self, *args, **kwargs):

        global res

        print("Clocking function {}".format(self.func.__name__))
        start_time = time.time()
        try:
            self.func(*args, **kwargs)
        except Exception as e:
            print(e)
            traceback.print_exc()
            #os.remove(db_path)
        finish = round(time.time() - start_time, 1)
        res.append(finish)
        print(finish)

        try:
            args[0].con.close()
        except AttributeError:
            print("aln not found")
            pass

        os.remove(db_path)


class GetResults(object):

    def __init__(self, func):
        self.func = func

    def __call__(self, *args, **kwargs):

        global res
        res = []

        self.func(*args, **kwargs)

        with open(res_path, "w") as fh:
            fh.write(",".join(map(str, res)))


@CleanUp
def read_alns(infiles):
    AlignmentList(infiles, sql_db=db_path)


@GetResults
def read_benchmark():

    for ds in ds_list:
        read_alns(ds)


@CleanUp
def convert_alns(aln_obj):
    aln_obj.write_to_file(["nexus"], output_dir=bench_dir)


@CleanUp
def convert_alns_int(aln_obj):
    aln_obj.write_to_file(["nexus"], output_dir=bench_dir, interleave=True)

@CleanUp
def concatenate_alns(aln):
    aln.concatenate()
    #aln.write_to_file(["nexus"], output_dir=bench_dir)

@CleanUp
def collapse(aln):
    aln.collapse(dest=bench_dir, use_main_table=True)

@CleanUp
def consensus(aln):
    aln.consensus("Soft mask", single_file=True, use_main_table=True)

@CleanUp
def filter_contain_taxa(aln, tx_lsit):
    aln.filter_by_taxa("Contain", tx_lsit)

@CleanUp
def filter_missing_data(aln):
    aln.filter_missing_data(50, 75)

@CleanUp
def filter_codon(aln):
    aln.filter_codon_positions([True, True, False])

@CleanUp
def filter_variable(aln):
    aln.filter_segregating_sites(0, 200)

@CleanUp
def filter_informative(aln):
    aln.filter_informative_sites(0, 200)

@CleanUp
def code_gaps(aln):
    aln.code_gaps()

@CleanUp
def rev_concatenation(aln, part_file):
    partition = data.Partitions()
    partition.read_from_file(part_file)
    aln = aln.alignments.values()[0]
    aln.set_partitions(partition)
    aln = aln.reverse_concatenate()

    # Post reverse ops
    print("Writing files")
    aln.write_to_file(["nexus"], output_dir=bench_dir)

@GetResults
def convert_benchmark():
    for p, ds in enumerate(ds_list):
        print("Profiling data set {}".format(p + 1))
        aln_obj = AlignmentList(ds, sql_db=db_path)
        # convert_alns(aln_obj)
        # convert_alns_int(aln_obj)
        # concatenate_alns(aln_obj)
        # collapse(aln_obj)
        # consensus(aln_obj)
        # filter_contain_taxa(aln_obj, flist[p])
        # filter_missing_data(aln_obj)
        # filter_codon(aln_obj)
        filter_variable(aln_obj)
        # filter_informative(aln_obj)

@GetResults
def reverse_conc():
    for p, (ds, part_file) in enumerate(zip(rev_ds_list, part_files)):
        print("Profiling data set {}".format(p + 1))
        aln_obj = AlignmentList(ds, sql_db=db_path)
        rev_concatenation(aln_obj, part_file[0])

# read_benchmark()
convert_benchmark()
# reverse_conc()

# @profile
# def read_a():
#     AlignmentList(ds_list[0], sql_db=db_path)
#
# read_a()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#
#  Copyright 2012 Unknown <diogo@arch>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.

import os
os.environ["KIVY_NO_ARGS"] = "1"

from kivy.uix.togglebutton import ToggleButton
from kivy.uix.button import Button
from kivy.uix.popup import Popup
from kivy.uix.label import Label
from kivy.core.window import Window
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.gridlayout import GridLayout
from kivy.uix.tabbedpanel import TabbedPanel
from kivy.uix.floatlayout import FloatLayout
from kivy.uix.textinput import TextInput
from kivy.uix.anchorlayout import AnchorLayout
from kivy.uix.spinner import Spinner
from kivy.uix.slider import Slider
from kivy.uix.treeview import TreeView
from kivy.uix.image import Image
from kivy.uix.widget import Widget
from kivy.uix.modalview import ModalView
from kivy.uix.filechooser import FileChooserListView
from kivy.core.text.markup import MarkupLabel as CoreMarkupLabel
from kivy.utils import get_hex_from_color
from kivy.properties import NumericProperty, StringProperty, BooleanProperty,\
    ObjectProperty
from kivy.uix.screenmanager import Screen
from kivy.logger import Logger
from kivy.graphics import Color, Line


import re
from weakref import ref
from os.path import (
    basename, join, sep, normpath, expanduser, abspath, pardir)
import sys

try:
    import data.resources.theme.default as tm
except ImportError:
    import trifusion.data.resources.theme.default as tm


class ShowcaseScreen(Screen):
    fullscreen = BooleanProperty(False)

    def add_widget(self, *args):
        if 'content' in self.ids:
            return self.ids.content.add_widget(*args)
        return super(ShowcaseScreen, self).add_widget(*args)


class StringInput(TextInput):
    """
    Modification of TextInput that only accepts integers
    """

    def insert_text(self, substring, from_undo=False):

        s = re.sub(r"\D", "", substring)

        return super(StringInput, self).insert_text(s, from_undo=from_undo)


# class HoverBehaviour(object):
#     """
#     Creates mouse over behaviour for widget
#     """
#
#     hovered = BooleanProperty(False)
#     border_point = ObjectProperty(None)
#     x1 = 0
#
#     def __init__(self, **kwargs):
#
#         self.register_event_type('on_enter')
#         self.register_event_type('on_leave')
#         #Window.bind(mouse_pos=self.on_mouse_pos)
#         Clock.schedule_interval(self.on_mouse_pos, 1)
#         super(HoverBehaviour, self).__init__(**kwargs)
#
#     def on_mouse_pos(self, *args):
#         if not self.get_root_window():
#             return  # do proceed if I'm not displayed <=> If have no parent
#         #pos = args[1]
#         pos = self.get_root_window().mouse_pos
#         self.x1+= 1
#
#         print(self.x1)
#         # Next line to_widget allow to compensate for relative layout
#         inside = self.collide_point(*self.to_widget(*pos))
#         if self.hovered == inside:
#             # We have already done what was needed
#             return
#         self.border_point = pos
#         self.hovered = inside
#         if inside:
#             self.dispatch('on_enter')
#         else:
#             self.dispatch('on_leave')
#
#     def on_enter(self):
#         pass
#
#     def on_leave(self):
#         pass


class LinkedLabel(Label):
    """
    Modification of label to reformat label for path linking
    """

    def __init__(self, **kwargs):
        super(LinkedLabel, self).__init__(**kwargs)

    @staticmethod
    def create_ref_label(text):
        """
        Modifies the original label by adding references to each directory in
        the path
        """

        if sys.platform in ["win32", "cygwin"]:
            s = p = ""
        else:
            s = p = sep

        # This will check if the first path_list element represents a windows
        # drive (like C:). If  so, it will add a "\" character, so that the
        # path is correctly constructed
        path_list = text.split(sep)
        if sys.platform in ["win32", "cygwin"] and len(path_list[0]) == 2\
                and path_list[0].endswith(":"):
            path_list[0] += "\\"

        for d in path_list:
            p = join(p, d)
            if p != sep:
                s += u"[ref={}]{}[/ref]".format(p, d) + sep

        return s[:-1]

    def texture_update(self, *largs):
        """Force texture recreation with the current Label properties.

        After this function call, the :attr:`texture` and :attr:`texture_size`
        will be updated in this order.
        """
        mrkup = self._label.__class__ is CoreMarkupLabel
        self.texture = None

        if (not self._label.text or (self.halign[-1] == 'y' or self.strip) and
                not self._label.text.strip()):
            self.texture_size = (0, 0)
            if mrkup:
                self.refs, self._label._refs = {}, {}
                self.anchors, self._label._anchors = {}, {}
        else:
            if mrkup:
                text = self.text
                text = self.create_ref_label(text)
                # we must strip here, otherwise, if the last line is empty,
                # markup will retain the last empty line since it only strips
                # line by line within markup
                if self.halign[-1] == 'y' or self.strip:
                    text = text.strip()
                self._label.text = ''.join(('[color=',
                                            get_hex_from_color(self.color),
                                            ']', text, '[/color]'))
                self._label.refresh()
                # force the rendering to get the references
                if self._label.texture:
                    self._label.texture.bind()
                self.refs = self._label.refs
                self.anchors = self._label.anchors
            else:
                self._label.refresh()
            texture = self._label.texture
            if texture is not None:
                self.texture = self._label.texture
                self.texture_size = list(self.texture.size)


class FileChooserL(FileChooserListView):
    """
    Modification of the FileChooserListView widget that fixes an issue of path
    update when clicking in the parent directory
    """

    def __init__(self, **kwargs):
        super(FileChooserL, self).__init__(**kwargs)
        self.register_event_type("on_double_click")
        self.register_event_type("on_dir_entry")

    def on_double_click(self):
        """
        Even triggered when double clicking a file
        """

        pass

    def on_dir_entry(self):
        """
        Event triggered when entering a directory
        """
        pass

    def open_entry(self, entry):
        """
        Modification of the open entry method so that when the entry.path is
        "../", the path is updated to the parent directory, instead of
        appending the entry.path
        """
        self.dispatch("on_dir_entry")
        try:
            # Just check if we can list the directory. This is also what
            # _add_file does, so if it fails here, it would also fail later
            # on. Do the check here to prevent setting path to an invalid
            # directory that we cannot list.
            self.file_system.listdir(entry.path)
        except OSError:
            entry.locked = True
        else:
            # If entry.path is to jump to previous directory, update path with
            # parent directory
            if entry.path == "../" or entry.path == "..\\":
                self.path = unicode(abspath(join(self.path, pardir)))
                self.selection = []
            else:
                self.path = unicode(join(self.path, entry.path))
                self.selection = []

    def entry_touched(self, entry, touch):
        '''(internal) This method must be called by the template when an entry
        is touched by the user.
        '''
        if (
            'button' in touch.profile and touch.button in (
                'scrollup', 'scrolldown', 'scrollleft', 'scrollright')):
            return False

        _dir = self.file_system.is_dir(entry.path)

        if not _dir and touch.is_double_tap:
            self.dispatch("on_double_click")

        if self.multiselect:
            if entry.path in self.selection:
                self.selection.remove(entry.path)
            else:
                if _dir and not self.dirselect:
                    self.open_entry(entry)
                    return
                self.selection.append(entry.path)
        else:
            if _dir and not self.dirselect:
                self.open_entry(entry)
                return
            self.selection = [entry.path, ]

    def _add_files(self, path, parent=None):
        path = expanduser(path)

        files = []
        fappend = files.append
        if sys.platform in ["win32", "cygwin"]:
            try:
                tdir = [str(x.encode("latin1")) for x in
                        self.file_system.listdir(path)]
            except UnicodeDecodeError:
                tdir = self.file_system.listdir(path)
            ldir = [unicode(x, "latin1") for x in tdir]
        else:
            ldir = self.file_system.listdir(path)
        for f in ldir:
            try:
                # In the following, use fully qualified filenames
                fappend(normpath(join(path, f)))
            except UnicodeDecodeError:
                Logger.exception('unable to decode <{}>'.format(f))
            except UnicodeEncodeError:
                Logger.exception('unable to encode <{}>'.format(f))
        # Apply filename filters
        files = self._apply_filters(files)
        # Sort the list of files
        files = self.sort_func(files, self.file_system)
        is_hidden = self.file_system.is_hidden
        if not self.show_hidden:
            files = [x for x in files if not is_hidden(x)]
        self.files[:] = files
        total = len(files)
        wself = ref(self)
        for index, fn in enumerate(files):

            def get_nice_size():
                # Use a closure for lazy-loading here
                return self.get_nice_size(fn)

            ctx = {'name': basename(fn),
                   'get_nice_size': get_nice_size,
                   'path': fn,
                   'controller': wself,
                   'isdir': self.file_system.is_dir(fn),
                   'parent': parent,
                   'sep': sep}
            entry = self._create_entry_widget(ctx)
            yield index, total, entry


class TreeViewM(TreeView):
    """
    Implementation of TreeViewNode widget to support multiple selection in
    ListView filechoosers
    """

    multiselect = BooleanProperty(False)
    """
    Boolean to indicate whether multiple nodes may be selected.
    """

    def __init__(self, **kwargs):
        super(TreeViewM, self).__init__(**kwargs)

    def select_node(self, node):
        '''Select a node in the tree.
        '''
        if node.no_selection:
            return
        else:
            if self._selected_node:
                self._selected_node.is_selected = False
            # Ignore node selection on multiselect. Selection is done in the
            # layout
            if self.multiselect:
                node.is_selected = False
            else:
                node.is_selected = True

        self._selected_node = node


class FileChooserM(FileChooserListView):
    """
    Modification of the FileChooserIconView widget that fixes n issue of path
    update when clicking in the parent directory and provides support for
    multiple file selection using Shift+Click. To achieve this, a few methods
    were added that listen to keyboard input in order to capture when the
    shift key is being pressed. These methods change the new shift attribute of
    the class, which is used when an entry is touched.

    The current Shift+Click implementation supports forward and backward
    selection from the last entry touched.
    """

    shift = False
    control = False

    # This attribute is a workaround of the "going back two par dirs" on
    # Windows. The problem is that, occasionally, three mouse inputs are
    # provided when double clicking. The third input is also registered as a
    # double tap and, therefore, the open_dir method is issue twice. This
    # attribute checks if the previous touch is False (previous touch was not
    # double tap) or True (previous touch was double tap), and only issues an
    # open dir when prev_touch is False
    prev_touch = False

    def __init__(self, **kwargs):
        super(FileChooserM, self).__init__(**kwargs)
        # Register new event that is triggered when entering a directory
        self.register_event_type("on_dir_entry")
        self.register_event_type("on_double_click")
        Window.bind(on_key_down=self.keyboard_listen)
        Window.bind(on_key_up=self.release_shift)

    def on_double_click(self):
        """
        Even triggered when double clicking a file
        """

        pass

    def on_dir_entry(self):
        """
        Event triggered when entering a directory
        """
        pass

    def keyboard_listen(self, *vals):
        """
        Listens to keyboard when a key is pressed. It is used to set the shift
        attribute to True when Shift is being pressed
        :param vals: keyboard input
        """

        key_code = vals[1]

        if key_code == 304:
            self.shift = True

        if key_code == 305:
            self.control = True

    def release_shift(self, *vals):
        """
        Listens to keyboard when a key is released. It is used to set the
        shift attribute to False when the Shift key is released.
        :param vals:
        :return:
        """

        key_code = vals[1]

        if key_code == 304:
            self.shift = False

        if key_code == 305:
            self.control = False

    def open_entry(self, entry):
        """
        Modification of the open entry method so that when the entry.path is
        "../", the path is updated to the parent directory, instead of
        appending the entry.path
        """
        self.dispatch("on_dir_entry")
        try:
            # Just check if we can list the directory. This is also what
            # _add_file does, so if it fails here, it would also fail later
            # on. Do the check here to prevent setting path to an invalid
            # directory that we cannot list.
            self.file_system.listdir(entry.path)
        except OSError:
            entry.locked = True
        else:
            # If entry.path is to jump to previous directory, update path with
            # parent directory
            if entry.path == "../" or entry.path == "..\\":
                self.path = unicode(abspath(join(self.path, pardir)))
                self.selection = []
            else:
                self.path = unicode(join(self.path, entry.path))
                self.selection = []

    def entry_released(self, entry, touch):
        '''
        Overides the entry_released method to prevent double entries when
        double clicking a directory without multiselect.
        '''
        pass


    def entry_touched(self, entry, touch):
        """
        (internal) This method must be called by the template when an entry
        is touched by the user. Supports Shift+Clicking for multiple selection
        """
        if (
            'button' in touch.profile and touch.button in (
                'scrollup', 'scrolldown', 'scrollleft', 'scrollright')):
            return False

        _dir = self.file_system.is_dir(entry.path)
        dirselect = self.dirselect

        if _dir and dirselect and touch.is_double_tap and not \
                self.shift and not self.prev_touch:
            self.open_entry(entry)
            self.prev_touch = touch.is_double_tap
            return

        # The 'not self.shift' fixes an incorrect behaviour of touch, where
        # shift+click sets is_double_tap to true, even when there is no
        # double click
        elif not _dir and touch.is_double_tap and not self.shift:
            self.dispatch("on_double_click")

        # Workaround for issue #151
        self.prev_touch = touch.is_double_tap

        if self.shift and self.selection:
            # Get index of last selection entry and current entry
            idx_selection = self.files.index(self.selection[-1])
            idx_current = self.files.index(entry.path)

            # If current entry is ahead of last selection, select files
            # going forward
            if idx_selection < idx_current:
                idx_s = idx_selection
                idx_f = idx_current
            else:
                idx_s = idx_current
                idx_f = idx_selection

        if self.multiselect:

            if not self.shift and not self.control:
                self.selection = []

            if entry.path in self.selection:
                # This will deselect multiple files when the shift key is down
                # while clicking
                if self.shift and self.selection:
                    for i in range(idx_s + 1, idx_f + 1):
                        f = self.files[i]
                        if f in self.selection:
                            self.selection.remove(f)
                else:
                    self.selection.remove(entry.path)
            else:
                if _dir and not self.dirselect:
                    self.open_entry(entry)
                    return
                # This will select multiple files when the shift key is down
                # while clicking
                if self.shift and self.selection:
                    for i in range(idx_s, idx_f + 1):
                        f = self.files[i]
                        if f not in self.selection:
                            self.selection.append(f)
                else:
                    self.selection.append(entry.path)
        else:
            if _dir and not self.dirselect:
                self.open_entry
                return
            self.selection = [entry.path, ]

    def _add_files(self, path, parent=None):
        path = expanduser(path)

        files = []
        fappend = files.append
        if sys.platform in ["win32", "cygwin"]:
            try:
                tdir = [str(x.encode("latin1")) for x in
                        self.file_system.listdir(path)]
            except UnicodeDecodeError:
                tdir = self.file_system.listdir(path)
            ldir = [unicode(x, "latin1") for x in tdir]
        else:
            ldir = self.file_system.listdir(path)
        for f in ldir:
            try:
                # In the following, use fully qualified filenames
                fappend(normpath(join(path, f)))
            except UnicodeDecodeError:
                Logger.exception('unable to decode <{}>'.format(f))
            except UnicodeEncodeError:
                Logger.exception('unable to encode <{}>'.format(f))
        # Apply filename filters
        files = self._apply_filters(files)
        # Sort the list of files
        files = self.sort_func(files, self.file_system)
        is_hidden = self.file_system.is_hidden
        if not self.show_hidden:
            files = [x for x in files if not is_hidden(x)]
        self.files[:] += files
        total = len(files)
        wself = ref(self)
        for index, fn in enumerate(files):

            def get_nice_size():
                # Use a closure for lazy-loading here
                return self.get_nice_size(fn)

            ctx = {'name': basename(fn),
                   'get_nice_size': get_nice_size,
                   'path': fn,
                   'controller': wself,
                   'isdir': self.file_system.is_dir(fn),
                   'parent': parent,
                   'sep': sep}
            entry = self._create_entry_widget(ctx)
            yield index, total, entry


class ModalView(ModalView):

    def __init__(self, **kwargs):

        super(ModalView, self).__init__(**kwargs)


class PopupMod(Popup):

    def __init__(self, **kwargs):

        super(Popup, self).__init__(**kwargs)


class CustomPopup(PopupMod):
    """
    Modification of Popup class with a few additional feature.

    .: The title does not wrap, but instead is shortened
    .: A custom background may be provided using the custom_background attribute
    """

    def __init__(self, **kwargs):

        super(CustomPopup, self).__init__(**kwargs)

        border_color = kwargs.get("border_color", tm.c_popup_background)

        label = self.children[0].children[-1]
        label.shorten = True
        label.shorten_from = "right"
        label.markup = True

        with self.canvas.after:
            Color(*border_color)
            self.line = Line(width=0.6,
                rectangle=[self.x, self.y, self.width, self.height])

        self.bind(pos=self.update_rect,
                  size=self.update_rect)

    def update_rect(self, *args):

        self.line.rectangle = [self.x, self.y, self.width, self.height]


class AutoCompTextInput(TextInput):
    """
    Modified widget of text input in which the tab key does not introduce a
    tabular space. This is meant to use with _auto_completion, in which the
    tab key serves as a keybinding
    """

    def insert_text(self, substring, from_undo=False):
        if substring == "\t":
            s = ""
        else:
            s = substring
        return super(AutoCompTextInput, self).insert_text(s,
                                                          from_undo=from_undo)


class SimpleModal(Widget):

    def __init__(self, **kwargs):
        super(SimpleModal, self).__init__(**kwargs)

    def on_touch_down(self, touch):
        return super(SimpleModal, self).on_touch_down(touch)

    def on_touch_up(self, touch):
        return super(SimpleModal, self).on_touch_up(touch)


class FancyDropDown(BoxLayout, SimpleModal):
    def __init__(self, **kwargs):
        super(FancyDropDown, self).__init__(**kwargs)

    def on_touch_down(self, touch):

        super(FancyDropDown, self).on_touch_down(touch)
        return True

    def on_touch_up(self, touch):
        super(FancyDropDown, self).on_touch_up(touch)
        return True


class SP_MoreOpts_Dialog(BoxLayout):
    def __init__(self, **kwargs):
        super(SP_MoreOpts_Dialog, self).__init__(**kwargs)

        self.ds_type = kwargs["ds_type"]


class PartitionsDialog(BoxLayout):
    """
    Custom layout for partition box when editing partitions
    """
    pass


class ModelSpinner(Spinner):
    """
    Custom Spinner that takes a background_normal argument to set the
    background
    """

    def __init__(self, **kwargs):
        super(ModelSpinner, self).__init__(**kwargs)

        try:
            self.background_normal = kwargs["background_normal"]
        except KeyError:
            pass


class MySlider(Slider):

    def __init__(self, **kwargs):
        super(MySlider, self).__init__(**kwargs)
        self.register_event_type("on_release")

    def on_release(self):
        pass

    def on_touch_up(self, touch):
        super(MySlider, self).on_touch_up(touch)
        if touch.grab_current == self:
            self.dispatch("on_release")
            return True


class CrunchData(BoxLayout):
    pass


class FileOverwriteDialog(BoxLayout):
    cancel = ObjectProperty()


class TFButton(Button, SimpleModal):

    def __init__(self, **kwargs):
        super(TFButton, self).__init__(**kwargs)

    def on_touch_down(self, touch):

        super(TFButton, self).on_touch_down(touch)

    def on_touch_up(self, touch):

        super(TFButton, self).on_touch_up(touch)


class TFButtonOff(Button):

    def __init__(self, **kwargs):
        super(TFButtonOff, self).__init__(**kwargs)


class TGToggleButton(ToggleButton):

    def __init__(self, **kwargs):
        super(TGToggleButton, self).__init__(**kwargs)


class ShortenToggleButton(ToggleButton):

    def __init__(self, **kwargs):
        super(ShortenToggleButton, self).__init__(**kwargs)


class ShortenButton(ToggleButton):

    def __init__(self, **kwargs):
        super(ShortenButton, self).__init__(**kwargs)


class ExportGraphics(BoxLayout):
    cancel = ObjectProperty(None)

    def __init__(self, **kwargs):
        super(ExportGraphics, self).__init__(**kwargs)

        kwargs["bookmark_init"](self.ids.bookmark_gl, self.ids.sv_mycomp,
                                self.ids.sd_filechooser, popup_level=2)


class AlignedTextInput(TextInput):
    halign = StringProperty('left')
    valign = StringProperty('top')

    DEFAULT_PADDING = 6

    def __init__(self, **kwargs):
        self.halign = kwargs.get("halign", "left")
        self.valign = kwargs.get("valign", "top")

        self.bind(on_text=self.on_text)

        super(AlignedTextInput, self).__init__(**kwargs)

    def on_text(self, instance, value):
        self.redraw()

    def on_size(self, instance, value):
        self.redraw()

    def redraw(self):
        """
        Note: This methods depends on internal variables of its TextInput
        base class (_lines_rects and _refresh_text())
        """

        self._refresh_text(self.text)

        max_size = max(self._lines_rects, key=lambda r: r.size[0]).size
        num_lines = len(self._lines_rects)

        px = [self.DEFAULT_PADDING, self.DEFAULT_PADDING]
        py = [self.DEFAULT_PADDING, self.DEFAULT_PADDING]

        if self.halign == 'center':
            d = (self.width - max_size[0]) / 2.0 - self.DEFAULT_PADDING
            px = [d * 1.1, d]
        elif self.halign == 'right':
            px[0] = self.width - max_size[0] - self.DEFAULT_PADDING

        if self.valign == 'middle':
            d = (self.height - max_size[1] * num_lines) / \
                2.0 - self.DEFAULT_PADDING
            py = [d * 1.1, d]
        elif self.valign == 'bottom':
            py[0] = self.height - max_size[1] * num_lines - \
                    self.DEFAULT_PADDING

        self.padding_x = px
        self.padding_y = py


class TableCell(AlignedTextInput):
    pass


class PlotChangeFilters(BoxLayout):
    pass


class ProcessExecutionProgress(BoxLayout):
    pass


class ProgressBox(BoxLayout):
    pass


class LoadSpinner(FloatLayout):
    pass


class LoadCounter(Label):
    pass


class ProgressFinished(Image):
    pass


class ProgressWaiting(Image):
    pass


class StatsLoading(BoxLayout):
    pass


class StatsSummary(BoxLayout):
    pass


class NoDataLabel(Label):
    pass


class GeneTable(BoxLayout):
    pass


class TableCell(Label):
    pass


class TableLine(BoxLayout):
    pass


class MoreTableBt(BoxLayout):
    pass


class InputType(BoxLayout):
    cancel = ObjectProperty()


class ExportGroupDialog(BoxLayout):
    cancel = ObjectProperty(None)


class InputTextDialog(BoxLayout):
    cancel = ObjectProperty(None)
    action = ObjectProperty(None)


class TFTextInput(TextInput):
    pass


class ExtSpinner(Spinner):
    pass


class ExtLabel(Label):
    pass


class BWCheck(BoxLayout):
    pass


class CheckProject(BoxLayout):
    cancel = ObjectProperty(None)


class BackButton(Button):
    pass


class ProjectOrtoBt(Button):
    pass


class ProjectProcBt(Button):
    pass


class SelectGeneDialog(BoxLayout):
    cancel = ObjectProperty()


class StatsToggleWgt(BoxLayout):
    avg_func = ObjectProperty(None)
    sp_func = ObjectProperty(None)


class StatsPlotToolbar(BoxLayout):
    pass


class PlotTriageDialog(BoxLayout):
    cancel = ObjectProperty(None)


class OrtoPlotToolbar(BoxLayout):
    pass


class OrthologySearchGrid(TabbedPanel):
    pass


class DescriptionBox(BoxLayout):
    # Attribute for number of proteins
    prot_txt = StringProperty()
    # Attribute for number of taxa
    taxa_txt = StringProperty()
    # Attribute for total number of orthologs
    ortholog_txt = StringProperty()


class GaugePlot(BoxLayout):
    # Attribute for Gauge plot top label
    txt = StringProperty()
    # Attribute for proportion for gauge plot. This proportion should range
    # between 0 and 1. It will be automatically adapted in the gauge plot
    proportion = NumericProperty()
    # Attribute for number of orthologs
    ortholog_num = StringProperty()


class OrtoSetFiltersDialog(BoxLayout):
    cancel = ObjectProperty(None)


class OrthoReportDialog(BoxLayout):
    cancel = ObjectProperty(None)


class OrthoGraphicReport(BoxLayout):
    pass


class FilterGraphicReport(BoxLayout):
    cancel = ObjectProperty(None)


class IndividualFilterWgt(BoxLayout):
    pass


class OrtoFilterDialog(BoxLayout):
    cancel = ObjectProperty(None)


class OrtoExecutionDialog(BoxLayout):
    cancel = ObjectProperty(None)


class OrtoProgressDialog(BoxLayout):
    pass


class ProteinFilterDialog(BoxLayout):
    cancel = ObjectProperty(None)


class MySQLDialog(BoxLayout):
    cancel = ObjectProperty(None)


class InflationDialog(BoxLayout):
    cancel = ObjectProperty(None)


class SidepanelToggle(ToggleButton):
    pass


class OptsGrid(GridLayout):
    pass


class StatsBox(BoxLayout):
    pass


class SequenceSimilarity(StatsBox):
    pass


class SegregatingSites(StatsBox):
    pass


class AFS(StatsBox):
    pass


class SizeDistribution(StatsBox):
    pass


class NucAAProportions(StatsBox):
    pass


class GeneOccupancy(StatsBox):
    pass


class MissingData(StatsBox):
    pass


class MissingOrto(StatsBox):
    pass


class CumulativeMissingOrto(StatsBox):
    pass


class LVCorrelation(StatsBox):
    pass


class TaxaDistribution(StatsBox):
    pass


class OutlierMissing(StatsBox):
    pass


class OutlierSegregating(StatsBox):
    pass


class OutlierSize(StatsBox):
    pass


class LoadMultipleDialog(BoxLayout):
    """
    A Filechooser widget in Icon view that allows multpiple file choosing
    """
    cancel = ObjectProperty(None)

    def __init__(self, **kwargs):
        super(LoadMultipleDialog, self).__init__(**kwargs)

        kwargs["bookmark_init"](self.ids.bookmark_gl, self.ids.sv_mycomp,
                                self.ids.sd_filechooser, popup_level=2)


class CloseBox(BoxLayout):
    """
    This is part of the taxa information popup. It contains the closing button
    """
    cancel = ObjectProperty(None)


class RemoveFloat(Button):
    """
    Simple (X) float button that can be associated with several root_window
    widgets for closing buttons
    """
    pass


class IconFloat(BoxLayout):
    pass


class CloseFloat(RemoveFloat):
    """
    A Simple (X) float button akin to the RemoveFloat widget. This widget was
    created specifically for popups and updates its positon according to the
    position of popus when resizing.
    """
    pass


class AboutDialog(BoxLayout):
    """
    Dialog with the About information of the App
    """
    pass


class DataSetTriageDialog(BoxLayout):
    cancel = ObjectProperty(None)


class WarningFloat(Label):
    """
    The general purpose unintruside warning float for errors and informations.
    This dialog is added to the root_window with fade in and fade out animations
    """
    pass


class FixButton(Button):
    pass


class DialogMCLFix(BoxLayout):
    cancel = ObjectProperty(None)


class DialogUSEARCHFix(BoxLayout):
    cancel = ObjectProperty(None)


class InfoPopup(BoxLayout):
    """
    Dialog for help texts that can be accessed for several options using the
    "?" button
    """
    cancel = ObjectProperty(None)


class FancyButton(Button):
    pass


class FancyMarker(Button):
    pass


class FancyMarkerPersist(Button):
    pass


class MouseOverLabel(Button):
    """
    General use mouse over label for diverse buttons
    """
    pass


class HseparatorFooter(BoxLayout):
    pass


class PropSwitcher(BoxLayout):
    pass


class OutlierFooter(BoxLayout):
    pass


class RevConcDialog(BoxLayout):
    """
    Reverse concatenation dialog
    """
    cancel = ObjectProperty(None)


class BtList(BoxLayout):
    cancel = ObjectProperty(None)


class InputList(BoxLayout):
    """
    Dialog with list of input files to select file for reverse concatenation
    """
    cancel = ObjectProperty(None)


class SideLabel(Label):
    """
    Mouseover label for side option buttons in the sidepanel
    """
    pass


class LoadMoreBt(AnchorLayout):
    """
    Custom button widget for for the "load more button" in the side panel
    """
    pass


class StatsMoreBt(AnchorLayout):
    pass


class PathLabel(LinkedLabel):
    """
    Dialog for the Label with the path for the main file chooser
    """
    pass


class PathText(AutoCompTextInput):
    """
    Dialog for the TextInput for the main file chooser that controls the path
     and inherits from the custom AutoCompTextInput for auto completion
    """

    def __init__(self, **kwargs):
        super(PathText, self).__init__(**kwargs)


class ExecutionDialog(BoxLayout):
    """
    The Execution dialog for Process operations
    """
    cancel = ObjectProperty(None)


class ProteomePopup(BoxLayout):
    """
    Informative popup for proteome files when clicking the "i" button in the
    side panel
    """
    cancel = ObjectProperty(None)


class LoadProgressDialog(BoxLayout):
    """
    Dialog for the progression dialog when loading files into the program
    """
    pass


class FilePopup(BoxLayout):
    """
    Class with a custom BoxLayout controlling the informative popup for the
     file buttons in the File tab of the side panel
    """
    cancel = ObjectProperty(None)


class TaxaEdit(BoxLayout):
    pass


class TaxaPopup(BoxLayout):
    """
    Class with a custom BoxLayout controlling the informative popup for the
     taxa buttons in the Taxa tab of the side panel
    """
    pass


class CheckDialog(BoxLayout):
    """
    Class controlling the layout of a general purpose dialog to check if the
    user wants of perform a certain action
    """
    cancel = ObjectProperty(None)


class WarningDialog(BoxLayout):
    """
    Class controlling the layout of a general purpose dialog to warn the user
    of certain events
    """
    cancel = ObjectProperty(None)


class PartitionActiveDialog(BoxLayout):
    cancel = ObjectProperty(None)


class LoadDialog(BoxLayout):
    """
    Class controlling a general purpose layout for loading additional files
    """
    cancel = ObjectProperty(None)

    def __init__(self, **kwargs):
        super(LoadDialog, self).__init__(**kwargs)

        kwargs["bookmark_init"](self.ids.bookmark_gl, self.ids.sv_mycomp,
                                self.ids.ld_filechooser)


class SaveDialog(FloatLayout):
    """
    Class controlling the layout of the save file dialog in the Process screen
    """
    save = ObjectProperty(None)
    text_input = ObjectProperty(None)
    cancel = ObjectProperty(None)

    def __init__(self, **kwargs):
        super(SaveDialog, self).__init__(**kwargs)

        kwargs["bookmark_init"](self.ids.bookmark_gl, self.ids.sv_mycomp,
                                self.ids.sd_filechooser, popup_level=2)


class SplitPartitions(BoxLayout):
    cancel = ObjectProperty(None)


class FormatDialog(BoxLayout):
    """
    Class controlling the layout of the output format dialog in the Process
    screen
    """
    cancel = ObjectProperty(None)


class TaxaFilterDialog(BoxLayout):
    cancel = ObjectProperty(None)


class CodonFilterDialog(BoxLayout):
    cancel = ObjectProperty(None)


class FilterDialog(BoxLayout):
    """
    Class controlling the layout of the gap/missing filtering options in the
    Process screen
    """
    cancel = ObjectProperty(None)


class VariationFilterDialog(BoxLayout):
    cancel = ObjectProperty(None)


class TextDialog(BoxLayout):
    """
    Class controlling a simple text input popup
    """
    cancel = ObjectProperty(None)


class FastaExtra(BoxLayout):
    cancel = ObjectProperty(None)


class NexusExtra(BoxLayout):
    cancel = ObjectProperty(None)


class PhylipExtra(BoxLayout):
    """
    Class controlling the dialog with extra options for phylip output format
    """
    cancel = ObjectProperty(None)


class IMa2Extra(BoxLayout):
    """
    Class controlling the dialog with extra options for the IMa2 output format
    """
    cancel = ObjectProperty(None)

class ProcessGeneral(GridLayout):
    """
    Class controlling the layout of the general options of the Process screen
    """
    pass


class AdditionalProcessContents(TabbedPanel):
    """
    Class controlling the layout of the additional options of the Process screen
    """
    pass


class TaxaGroupDialog(BoxLayout):
    """
    Class controlling the layout of the taxa group creation dialogue in the
    side panel
    """
    cancel = ObjectProperty(None)


class ZorroDialog(BoxLayout):
    """
    Class controlling the layout of the ZORRO operation dialog
    """
    cancel = ObjectProperty(None)


__author__ = "Diogo N. Silva"



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#  Copyright 2012 Unknown <diogo@arch>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.


################################################################################

"""
This module contains the text content for the informative popups that can be
accessed by clicking info (?) buttons throughout the app. The text content
will be stored in a single dictionary variable, whose format should be the
following:
   ..: key: string id of the informative content
   ..: value: list containing the actual informative content
      ..: 1st element: string with the title
      ..: 2nd element: string with text body

Both list elements will be kivy Label objects that support markdown syntax
"""

informative_storage = {
    # Orthology
    "orthology_filters":
        ["[b]Orthology filters - Help[/b]",
         "These filters are applied at the end of the Orthology Search "
         "operation for each"
         " specified inflation value "
         "and determine which ortholog clusters are selected according to "
         "their number of gene copies and taxa. They can be later modified in"
         " the Orthology Explore tab to evaluate their effect on the final "
         "number of ortholog clusters."
         "\n\n- [b]Maximum number of genes copies[/b]: "
         "Sets the maximum number of gene copies in each ortholog cluster. To "
         "remove any upper limit on the number of gene copies, set this value "
         "to 0.\n\n- [b]Minimum number of taxa[/b]: Sets the minimum number of "
         "taxa that must be present in an ortholog cluster. Values can be "
         "either absolute or in proportion."],
    "orthology_output":
        ["[b]Orthology output directory - Help[/b]",
         "Sets the directory where all orthology search results will be "
         "stored. The directory structure will follow these general rules:"
         "\n\n- [b]'backstage_files'[/b]: Intermediate files that are "
         "necessary "
         "during the execution of orthology search. Can be useful for error "
         "checking. Also contains the protein data base file used for the "
         "all-vs-all search (goodProteins_db, by default)\n\n- ["
         "b]'Orthology_results'[/b]: "
         "Contains the final results of the orthology search, including the "
         "final group files. For each inflation parameter that was selected, "
         "a directory will be created with the results for the corresponding "
         "parameter value and containing the sequence files of the ortholog "
         "clusters after applying the orthology filters."],
    "database_name":
        ["[b]Database name - Help[/b]",
         "This option will set the name of the protein sequence database "
         "that will be used to performed all-vs-all searches with USEARCH. "
         "This database can also be subsequently used to convert OrthoMCL "
         "group files into protein sequences, by fetching the sequence "
         "references of the group clusters from the database file."],
    "inflation":
        ["[b]Inflation - Help[/b]",
         "Inflation is a parameter of the MCL algorithm that regulates "
         "the tightness of the ortholog clusters, with conceivable values "
         "ranging from 1.1 to 10, but rarely useful at the extremes. Using "
         "higher values (higher tightness) results in an increase of ortholog"
         " clusters but reduces the number of sequences included in each "
         "cluster. Conversely, lower values result in the inclusion of more "
         "sequences in fewer groups. According to the [u][ref=http://www.ncbi"
         ".nlm.nih.gov/pmc/articles/PMC403725/]OrthoMCL paper[/ref][/u], "
         "coarsed-grained clustering (1.5 value) provides sufficient "
         "tightness for identifying coherent ortholog clusters and appears to "
         "strike a balance between sensitivity and sensibility. However, "
         "they also note that this parameter may have little impact on the "
         "final number of clusters. A sensible approach would be to test "
         "several values and evaluate the sensibility of the data set to this "
         "parameter."],
    # Process
    "revert_concatenation":
        ["[b]Revert concatenation - Help[/b]",
         "The [b]revert concatenation[/b] option allows for the conversion of a"
         " single file with defined partitions into individual files for each "
         "partition. Partitions can be specified in two ways:\n\n- A "
         "partition file may be imported using the 'Select partition file' in "
         "the revert concatenation settings. Supported formats for the "
         "partition file include the Nexus charset block and a RAxML "
         "partition file;\n\n- Using defined partitions in "
         "the 'Partitions' tab in the side panel. Note that some input file "
         "formats, such as Nexus, may already contain defined partitions.\n\n"
         " This option takes precedence over all other Process' options, "
         "which means that all secondary operations (Collapse, Consensus, "
         "Filter, etc.) are applied to the individual files that result from "
         "this option."],
    "process_dataset":
        ["[b]Data set selection - Help[/b]",
         "The [b]Data set[/b] option offers a convenient way to perform "
         "operations on specific sets of taxa and/or files. User specified "
         "groups can be selected by toggling the taxon or file buttons in 'Menu"
         " > Open/View data' and selecting the 'Active taxa/files' option or "
         "by creating taxa groups in 'Menu > Dataset Groups' and selecting the "
         "name of the corresponding group."],
    "zorro":
        ["[b]ZORRO weights - Help[/b]",
         "(Concatenation only)\n\nThis option allows the user to"
         " concatenate alignment weight files that are generated by the "
         "ZORRO software [u][ref=http://journals.plos.org/plosone/article?id=10"
         ".1371/journal.pone.0030288](Wu M et al. 2012, PLoS ONE 7(1): "
         "e30288)[/ref][/u].\n\nIn order to maintain correspondence between "
         "the alignment and weight files, the weight file names should begin "
         "with the name of the corresponding alignment file and end with a "
         "suffix that must be provided. For example, if the files:\n\n "
         "-alignment_a.fas\n -alignment_b.fas\n\nare to be concatenated, "
         "then the corresponding zorro weight files should be:\n\n "
         "-alignment_a_zorro.txt\n -alignment_b_zorro.txt\n\nwhere '_zorro' "
         "is the suffix."],
    "collapse":
        ["[b]Collapse - Help",
         "The [b]Collapse[/b] option will merge all identical sequences into "
         "single haplotypes, so that the final alignment  contains only "
         "unique sequences. As a consequence, the taxa names will be "
         "replaced by haplotype names, which can be partially "
         "determined by the  user using the [b]Haplotype prefix[/b] option, "
         "and the correspondence between haplotype names and taxa names with "
         "the same sequence will be written into an *.haplotypes file."],
    "filter":
        ["[b]Gap/Missing Filter - Help",
         "The [b]Gap/Missing data[/b] option will perform two main tasks:\n\n "
         "[b]1. Within alignments:\n[/b]"
         "- Replacement of the gap symbols by missing data symbols at the "
         "beginning and end of the alignments created by the alignment "
         "software;\n"
         "- Removal of alignment columns containing a proportion of gaps "
         "and/or missing data higher than the thresholds specified using "
         "the [b]Gap threshold[/b] and [b]Missing data threshold[/b] sliders."
         "The [b]gap threshold[/b] refers only to gap characters found in the "
         "middle of the alignment. The [b]missing data threshold[/b] refers "
         "to the combination of gap and missing data characters found in any "
         "column of the alignment.\n\n"
         "[b]2. Among multiple alignments:[/b]\n"
         "- Alignments can be filtered based on the proportion of taxa "
         "represented. The total number of taxa is gathered from the full "
         "data set. If, for example, the minimum taxa representation value is "
         "set to 50%, only alignments with at least 50% of the full data "
         "set's taxa will be processed."],
    "taxa_filter":
        ["Taxa Filter - Help",
         "The [b]Taxa Filter[/b] option allows alignments to be filtered "
         "depending on whether they contain or exclude a given set of taxa. "
         "When choosing the filter mode  '[b]Contain[/b]', only those "
         "alignments containing at least the specified taxa in the taxa "
         "group will be saved. When choosing the '[b]Exclude[/b]' filter "
         "mode, alignments that contain [i]all[/i] the "
         "taxa specified in the taxa group will [i]not[/i] be saved."],
    "codon_filter":
        ["Codon Filter - Help",
         "The [b]Codon filter[/b] option is only available for nucleotide "
         "sequences, where it allows the filtering of codon positions in "
         "the alignments. By default, all positions are set to be saved but "
         "any combination can be specified to be filtered out."],
    "variation_filter":
        ["Variation Filter - Help",
         "The [b]variation filter[/b] option allows alignments to be "
         "filtered according their amount or type of sequence variation.\n\n["
         "b]Variable sites[/b] refer to any alignment column that contains at "
         "least one variante.\n\n[b]Informative sites[/b] refer to variable "
         "sites whose minimum frequency variant is present in at least "
         "in two taxa"],
    "consensus":
        ["Alignment consensus - Help",
         "The alignment consensus option will merge all sequences in an "
         "alignment into a single consensus sequence. Variation in alignment "
         "columns can be either converted into the corresponding IUPAC symbols,"
         " soft masked or removed from the final consensus"],
    "consensus_mode":
        ["Consensus variation handling - Help",
         "The consensus mode option determines how variation in an alignment "
         "will be handled to create a consensus sequence. The options are:"
         "\n\n[b]IUPAC:[/b] Converts variable sites using the IUPAC ambiguity"
         " code.\n\n[b]Soft mask:[/b] Replaces variable sites with missing "
         "data symbol.\n\n[b]Remove:[/b] Removes variable sites.\n\n"
         "[b]First sequence:[/b] Usesthe first sequence in each alignment"],
    "gcoder":
        ["[b]Gap coding - Help[/b]",
         "(Nexus output format only)\n\n"
         "This option allows for the codification of gap patterns as a binary "
         "matrix that is appended at the end of the sequences. The "
         "codification of the gaps follows the method selected in the '[b]Gap "
         "coding method[/b]' option. "],

    # Statistics
    "info_sequence_size":
        ["[b]Distribution of sequence size - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Per species; Average["
         "/b]\n\n"
         "[color=37abc8ff][b]Per species[/color][/b]: Plots the "
         "distribution of average sequence size (excluding sites with gaps "
         "or missing data)\n\n"
         "[b]y-axis:[/b] Sequence size in nucleotides (DNA) or residues ("
         "Protein)\n"
         "[b]x-axis:[/b] Taxon name\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Plots the distribution of "
         "the average sequence size (excluding sites with gaps or missing "
         "data) for the entire active data set.\n\n"
         "[b]y-axis:[/b] Absolute frequency of alignments\n"
         "[b]x-axis:[/b] Sequence size in nucleotides (DNA) or residues ("
         "Protein)"],
    "info_nucleotide_proportion":
        ["[b]Nucleotide or residue proportion - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Per species; Average["
         "/b]\n\n"
         "[color=37abc8ff][b]Per species[/color][/b]: Stacked bar "
         "plot with the proportions of each nucleotide (DNA) or residue ("
         "Protein) for each species in the active data set. A sorted color "
         "legend is provided above the plot.\n\n"
         "[b]y-axis:[/b] Proportion of nucleotide or residue\n"
         "[b]x-axis:[/b] Taxon name\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Bar plot "
         "with the averaged proportions for each nucleotide or residue across "
         "the active data set.\n\n"
         "[b]y-axis:[/b] Proportion\n"
         "[b]x-axis:[/b] Nucleotide or residue"],
    "info_taxa_frequency":
        ["[b]Distribution of taxa frequency - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Average[/b]\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Histogram with "
         "the frequency of alignments containing a given number of taxa. This "
         "plot can be useful in large data sets to evaluate the number of "
         "genes containing many or few taxa. Dense data sets (few missing "
         "taxa) tend to have distributions shifted to the right side, "
         "while sparse data sets tend to have distributions shifted to the "
         "left side.\n\n"
         "[b]y-axis:[/b] Absolute frequency of alignments\n"
         "[b]x-axis:[/b] Number of taxa present"],
    "pairwise_seq_similarity":
        ["[b]Pairwise sequence similarity - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Single gene; "
         "Per species; Average[/b]\n\n"
         "[color=37abc8ff][b]Single gene[/color][/b]: Sliding "
         "window line plot with the average sequence similarity across the "
         "length of the selected alignment.\n\n"
         "[b]y-axis:[/b] Sequence similarity, in percentage\n"
         "[b]x-axis:[/b] Alignment nucleotide/residue position\n\n"
         "[color=37abc8ff][b]Per species[/color][/b]: Creates a triangular "
         "heat map matrix with the sequence similarity values for each "
         "pair-wise combination of taxa. Sequence similarity values are color"
         " coded according to the color bar at the right of the plot.\n\n"
         "[b]y-axis:[/b] Taxa names\n"
         "[b]x-axis:[/b] Taxa names\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Plots an histogram with the"
         " distribution of sequence similarity values across the "
         "active data set. Each entry in the histogram corresponds to "
         "average sequence similarity of one alignment.\n\n"
         "[b]y-axis[/b] Absolute frequency of alignments\n"
         "[b]x-axis[/b] Sequence similarity, in percentage"],
    "segregating_sites":
        ["[b]Segregating sites - Help",
         "[color=37abc8ff][b]Available options:[/color] Single gene; "
         "Per species; Average[/b]\n\n"
         "[color=37abc8ff][b]Single gene[/color][/b]: Sliding "
         "window line plot with the number of segregating sites across the "
         "length of the selected alignment.\n\n"
         "[b]y-axis:[/b] Number of segregating sites\n"
         "[b]x-axis:[/b] Alignment nucleotide/residue position\n\n"
         "[color=37abc8ff][b]Per species[/color][/b]: Triangular "
         "heat map matrix with the number of segregating sites for each "
         "pair-wise combination of taxa. The number of segregating sites are "
         "color coded according to the color bar at the right of the plot.\n\n"
         "[b]y-axis:[/b] Taxa names\n"
         "[b]x-axis:[/b] Taxa names\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Plots an histogram with the"
         " distribution of the number of segregating sites, in absolute "
         "values or percentage, across the "
         "active data set. Each entry in the histogram corresponds to "
         "averaged number of segregating sites in one alignment.\n\n"
         "[b]y-axis:[/b] Absolute frequency of alignments\n"
         "[b]x-axis:[/b] Number of segregating sites, in absolute or "
         "percentage"],
    "len_pol_correlation":
        ["[b]Alignment length/polymorphism correlation[/b]",
         "[color=37abc8ff][b]Available options:[/color] Average[/b]\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Calculates the "
         "non-parametric spearman's rank correlation coefficient between the "
         "alignment length and number of informative sites for each "
         "alignment across the active data set. A "
         "scatter plot depicting the relationship between these two variables"
         " is generated along with the best fit line. The correlation "
         "coeficient and p-value are provided in the top right area of the "
         "plot.\n\n"
         "[b]x-axis:[/b] Number of informative sites\n"
         "[b]y-axis:[/b] Alignment length"],
    "afs":
        ["[b]Allele frequency spectrum - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Single gene, Average["
         "/b]\n\n"
         "[b](Nucleotide sequence only)[/b]\n\n"
         "[color=37abc8ff][b]Single gene[/color][/b]: Plots the "
         "distribution of allele frequencies for a single gene.\n\n"
         "[b]y-axis:[/b] Frequency of occurrence\n"
         "[b]x-axis:[/b] Derived allele frequency\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Creates the distribution of "
         "allele frequencies across the active data set. For each alignment, "
         "the distribution of the derived allele frequency is calculated and "
         "compiled into a single data matrix that depicts the distribution of "
         "allele frequencies for the entire data set.\n\n"
         "[b]y-axis:[/b] Frequency of occurrence\n"
         "[b]x-axis:[/b] Derived allele frequency\n"],
    "gene_occupancy":
        ["[b]Gene occupancy - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Average[/b]\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Plots an interpolation "
         "matrix that scores the presence/absence of each taxon for every "
         "alignment in the active data set. Each taxa is depicted as a line "
         "in the y-axis and each alignment as a column in the x-axis. "
         "Whenever a taxon is absent in a given alignment, the coordinates of "
         "this taxon-alignment combination receive a score of 0 and a blank "
         "space is represented. Conversely, if the taxa in present in a given "
         "alignment, the corresponding coordinate will receive a score of 1 "
         "and a black space is represent. Sparse data sets with taxa missing "
         "in several alignments will tend to have more white spaces. "
         "Conversely, dense data sets with very few missing taxa will have "
         "fewer white spaces.\n\n"
         "[b]y-axis:[/b] Taxa index, each line represents one taxon\n"
         "[b]x-axis:[/b] Alignment index, each column represents an alignment"],
    "missing_taxa":
        ["[b]Distribution of missing taxa - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Per species, Average["
         "/b]\n\n"
         "[color=37abc8ff][b]Per species[/color][/b]: Plots the distribution "
         "of the number of genes missing for each taxon.\n\n"
         "[b]y-axis:[/b] Frequency, number of genes missing for a given taxon\n"
         "[b]x-axis:[/b] Taxa names.\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Plots the distribution of "
         "the number of taxa missing for each alignment across the entire "
         "active data set.\n\n"
         "[b]y-axis:[/b] Frequency, number of occurences\n"
         "[b]x-axis:[/b] Number of taxa missing"],
    "missing_data":
        ["[b]Distribution of missinf data - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Per species, Average["
         "/b]\n\n"
         "[color=37abc8ff][b]Per species[/color][/b]: Stacked bar "
         "plot with the proportions of gaps, missing data and effective data "
         "for each taxon averaged over the active alignments. In cases "
         "where a given taxon is missing from an alignment, this will score "
         "100% of missing data for that alignment.\n\n"
         "[b]y-axis:[/b] Proportion\n"
         "[b]x-axis:[/b] Taxa names\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Smooth line "
         "histogram for three data categories: Gaps, Missing data "
         "and effective Data. For each alignment in the active data set, "
         "the proportion for each of these categories is calculated and then "
         "compiled into three data matrices. Each plot will then contain "
         "the distribution of the corresponding category across the entire "
         "active data set. As in the 'Per species' plot, taxa that are "
         "missing from a given alignment will score 100% of missing data for "
         "that alignment. Data sets "
         "with little missing data will tend to have 'Data' distributions "
         "abutting 100% values and 'Gaps' and 'Missing data' distributions "
         "abutting 0%. Increasing the amount of missing data will erase this "
         "pattern and tend to homogenise all three distribution or even to "
         "invert the trend.\n\n"
         "[b]y-axis: [/b]Frequency, number of genes\n"
         "[b]x-axis: [/b]Proportion, for each of the three categories"],
    "cumulative_missing":
        ["[b]Cumulative distribution of missing taxa - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Average[/b]\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Plots the number of genes "
         "available when applying a consecutive minimum taxa representation "
         "filters. This range spans from 0% (All values of missing data "
         "allowed) up to 100% (Alignments with no missing data) with steps "
         "of 5%. This plot can be useful when trying to decide the "
         "the best value for the minimum taxa representation filter that "
         "minimizes missing data and maximized effective data.\n\n"
         "[b]y-axis:[/b] Frequency, number of alignments\n"
         "[b]x-axis:[/b] Minimum taxa representation threshold, in percentage"],
    "outliers_missing":
        ["[b]Missing data outliers - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Per Species, Average["
         "/b]\n\n"
         "[b]All outlier detection methods employed in TriFusion use a "
         "slightly modified Median Absolute Deviation (MAD) method to find "
         "outliers. As the name implies, this method relies on the "
         "calculation of the median for the data set and then evaluate the "
         "distance from each data point to this value. The recommended "
         "threshold for this distance (above which a data point is considered "
         "an outlier) is 3.5, which is roughly equivalent to 3.5 standard "
         "deviations. The slight modification of the method is the inclusion "
         "of a consistency constant, which ensures that MAD provides a good "
         "estimate of the standard deviation independently of the sample "
         "size.[/b]\n\n"
         "[color=37abc8ff][b]Per species[/color][/b]: Smooth line "
         "distribution of the amount of missing data for each taxon across the "
         "active data set. Data points will be based on the proportion of "
         "missing data symbols out of the possible total. For example, in an "
         "alignment with three taxa, each with 100 sites, the total "
         "possible missing data is 300 (100 * 3). Taxa that are absent from "
         "a given alignment are ignored, in order to avoid biasing outlier "
         "detection to missing taxa instead of missing data. Data points "
         "represent individual taxa and are depicted in a rug-like style near "
         "the x-axis line. Outliers are marked with red dots.\n\n"
         "[b]y-axis:[/b] Frequency, number of occurrences\n"
         "[b]x-axis:[/b] Proportion of missing data\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Smooth line "
         "distribution of the average missing data for each alignment across "
         "the active data set. Data points represent individual alignments "
         "and are depicted in a rug-like style near the x-axis line. Outliers "
         "are marked with red dots.\n\n"
         "[b]y-axis:[/b] Frequency, number of occurrences\n"
         "[b]x-axis:[/b] Proportion of missing data"],
    "outliers_segregating":
        ["[b]Segregating sites outliers - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Per Species, Average["
         "/b]\n\n"
         "[b]All outlier detection methods employed in TriFusion use a "
         "slightly modified Median Absolute Deviation (MAD) method to find "
         "outliers. As the name implies, this method relies on the "
         "calculation of the median for the data set and then evaluate the "
         "distance from each data point to this value. The recommended "
         "threshold for this distance (above which a data point is considered "
         "an outlier) is 3.5, which is roughly equivalent to 3.5 standard "
         "deviations. The slight modification of the method is the inclusion "
         "of a consistency constant, which ensures that MAD provides a good "
         "estimate of the standard deviation independently of the sample "
         "size.[/b]\n\n"
         "[color=37abc8ff][b]Per species[/color][/b]: Smooth line "
         "distribution of the average pair-wise proportion of segregating "
         "sites of a single taxon when compared with every other taxon. "
         "Taxa outliers will be "
         "identified when a given taxon has an average excess or lack of "
         "segregating sites with every other taxon. Alignment columns "
         "containing gaps or missing data are ignored. Data points represent"
         " individual taxa and are depicted in a rug-like style near the "
         "x-axis line. Outliers are marked with red dots.\n\n"
         "[b]y-axis:[/b] Frequency, number of occurrences\n"
         "[b]x-axis:[/b] Proportion of segregating sites\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Smooth line "
         "distribution of the proportion of segregating sites for each "
         "alignment across the active data set. Alignment outliers will be"
         " identified when an alignment has an excess of lack of segregating "
         "sites compared to the remaining alignments. Alignment columns "
         "containing gaps or missing data are ignored. Data points represent "
         "individual alignments and are depicted in a rug-like style near "
         "the x-axis line. Outliers are marked with red dots.\n\n"
         "[b]y-axis:[/b] Frequency, number of occurrences\n"
         "[b]x-axis:[/b] Proportion of segregating sites"],
    "outliers_size":
        ["[b]Sequence size outliers - Help[/b]",
         "[color=37abc8ff][b]Available options:[/color] Per Species, Average["
         "/b]\n\n"
         "[b]All outlier detection methods employed in TriFusion use a "
         "slightly modified Median Absolute Deviation (MAD) method to find "
         "outliers. As the name implies, this method relies on the "
         "calculation of the median for the data set and then evaluate the "
         "distance from each data point to this value. The recommended "
         "threshold for this distance (above which a data point is considered "
         "an outlier) is 3.5, which is roughly equivalent to 3.5 standard "
         "deviations. The slight modification of the method is the inclusion "
         "of a consistency constant, which ensures that MAD provides a good "
         "estimate of the standard deviation independently of the sample "
         "size.[/b]\n\n"
         "[color=37abc8ff][b]Per species[/color][/b]: Smooth line "
         "distribution of the average sequence size for each taxon. The "
         "sequence size for each taxon excludes gaps and missing data. Taxa "
         "outliers will be identified when a given taxon has sequences much "
         "larger or smaller than the median of the active data set. Data "
         "points represent"
         " individual taxa and are depicted in a rug-like style near the "
         "x-axis line. Outliers are marked with red dots.\n\n"
         "[b]y-axis:[/b] Frequency, number of occurrences\n"
         "[b]x-axis:[/b] Sequence size in nucleotides/residues\n\n"
         "[color=37abc8ff][b]Average[/color][/b]: Smooth line "
         "distribution of the average sequence size for each alignment. The "
         "sequence size for each alignment excludes gaps and missing data. "
         "Outlier alignments will be identified when the average sequence "
         "size of an alignment is much larger or smaller than the median of "
         "the active data set. Data points represent "
         "individual alignments and are depicted in a rug-like style near "
         "the x-axis line. Outliers are marked with red dots.\n\n"
         "[b]y-axis:[/b] Frequency, number of occurrences\n"
         "[b]x-axis:[/b] Sequence size in nucleotides/residues"]
}

orthology_storage = {
    "Taxa distribution": "Distribution of the number of taxa across clusters",
    "Taxa coverage": "Shows available and missing ortholog data per taxon",
    "Gene copy distribution": "Distribution of gene copy numbers across "
        "clusters",
    "Taxa gene copies": "Cumulative number of multiple gene copies per taxon"
}

# Informative text for Orthology plots. Information for each plot is provided
#  as a tuple of three elements: (title, category, figure name, description)
orthology_plots = [
    ("Distribution of maximum gene copies",
     "Gene focused",
     "bar_genecopy_distribution.png",
     "Displays the number of orthologs in function of the maximum number of "
     "gene copies. Here, the number of gene copies in each ortholog cluster is "
     "obtained from the taxa with the highest number of gene copies. "
     "Therefore, if an ortholog cluster with 10 taxa has a single copy "
     "for 9 taxa and 10 copies for 1 taxon, that ortholog cluster scores a "
     "value of 10. "
     "Orthologs with a single gene copy effectively represent single "
     "copy genes. We recommend creating this plot without applying a maximum "
     "gene copy filter in order to get the full distribution of the data set."
     "<br/><br/>"
     "<b>x-axis:</b> Number of maximum gene copies<br/>"
     "<b>y-axis:</b> Frequency of ortholog groups"),
    ("Species distribution",
     "Species focused",
     "bar_species_distribution.png",
     "Displays the number of orthologs in function of the exact number of "
     "unique taxa represented in a given ortholog. Here, taxa represented by "
     "multiple copies are scored only once but the range of the number of "
     "taxa is determined by the minimum taxa representation filter. In the "
     "absence of a minimum taxa representation filter, it is entirely "
     "possible to have groups with a single taxon, which represent"
     "clusters of recent paralogs for that taxon.<br/><br/>"
     "<b>x-axis:</b> Number of taxa<br/>"
     "<b>y-axis:</b> Frequency of ortholog groups"),
    ("Data coverage per species",
     "Species focused",
     "bar_species_coverage.png",
     "Displays the amount of available and missing ortholog groups for each "
     "taxon. The number of available genes is displayed as dark blue bars, "
     "while the number of missing genes is displayed as light blue bars. Given "
     "the fact that we are dealing with non-aligned data, this will "
     "simply score the number of genes available for a given taxa in relation "
     "to the total number of ortholog cluster for the current filters. "
     "(Therefore, these amounts can also be viewed as proportions.) This "
     "analysis will NOT take into account the number of gaps that will be "
     "generated after the alignment procedure. The horizontal dashed line "
     "represents the mean available data for the data set.<br/><br/>"
     "<b>x-axis:</b> Taxon<br/>"
     "<b>y-axis:</b> Frequency of ortholog groups"),
    ("Distribution of gene copy per taxa",
     "Species focused",
     "bar_genecopy_per_species.png",
     "Displays the number of gene copies for each taxon. Here, only ortholog "
     "clusters with multiple copies for a given taxa are taken into account. "
     "If a taxon has only single copies in the entire data set, it scores 0."
     "<br/><br/>"
     "<b>x-axis:</b> Taxon<br/>"
     "<b>y-axis:</b> Number of gene copies (above single copy)")
]

__author__ = "Diogo N. Silva"



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#
#  Copyright 2012 Unknown <diogo@arch>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.

try:
    from process.error_handling import InputError, EmptyAlignment
except ImportError:
    from trifusion.process.error_handling import InputError, EmptyAlignment

import traceback
import shutil
import os
import time
import sys
from collections import OrderedDict

dna_chars = ["a", "t", "g", "c"]

aminoacid_table = OrderedDict({"a": ["Alanine", "nonpolar", "neutral"],
                               "r": ["Arginine", "polar", "positive"],
                               "n": ["Asparagine", "polar", "neutral"],
                               "d": ["Aspartate", "polar", "negative"],
                               "c": ["Cysteine", "nonpolar", "neutral"],
                               "e": ["Glutamate", "polar", "negative"],
                               "q": ["Glutamine", "polar", "neutral"],
                               "g": ["Glycine", "nonpolar", "neutral"],
                               "h": ["Histidine", "polar", "neutral"],
                               "i": ["Isoleucine", "nonpolar", "neutral"],
                               "l": ["Leucine", "nonpolar", "neutral"],
                               "k": ["Lysine", "polar", "positive"],
                               "m": ["Methionine", "nonpolar", "neutral"],
                               "f": ["Phenylalanine", "nonpolar", "neutral"],
                               "p": ["Proline", "nonpolar", "neutral"],
                               "s": ["Serine", "polar", "neutral"],
                               "t": ["Threonine", "polar", "neutral"],
                               "w": ["Tryptophan", "nonpolar", "neutral"],
                               "y": ["Tyrosine", "polar", "neutral"],
                               "v": ["Valine", "nonpolar", "neutral"],
                               "u": ["Selenocysteine", "", ""],
                               "o": ["Pyrrolysine", "", ""],
                               "x": ["Missing", "", ""]})

iupac = {"ag": "r", "ct": "y", "cg": "s", "at": "w", "gt": "k", "ac": "m",
         "cgt": "b", "agt": "d", "act": "h", "acg": "v", "acgt": "n"}

iupac_rev = {"v": "acg", "r": "ag", "m": "ac", "s": "cg", "d": "agt",
             "b": "cgt", "n": "acgt", "h": "act", "y": "ct", "w": "at",
             "k": "gt"}

iupac_conv = {"v": "acg", "r": "ag", "m": "ac", "s": "cg", "d": "agt",
              "b": "cgt", "n": "acgt", "h": "act", "y": "ct", "w": "at",
              "k": "gt", "a": "aa", "t": "tt", "c": "cc", "g": "gg"}


class CleanUp(object):

    def __init__(self, func):
        self.func = func
        self.temp_dir = ".trifusion-temp"
        self.idx = 0 if self.func.__name__ == "main_parser" else 2

    def __call__(self, *args):

        try:
            start_time = time.time()
            self.func(*args)
            if not args[0].quiet:
                print_col("Program execution successfully completed in %s "
                          "seconds" %
                          (round(time.time() - start_time, 2)), GREEN,
                          self.idx)
        except KeyboardInterrupt:
            # Removing temporary directory, if any
            if os.path.exists(self.temp_dir):
                shutil.rmtree(self.temp_dir)
            print_col("Interrupting, by your command", RED, self.idx)
        # The broad exception handling is used to remove the temporary
        # directory under any circumstances
        except Exception as e:
            print(e)
            traceback.print_exc()
            # Removing temporary directory, if any
            if os.path.exists(self.temp_dir):
                shutil.rmtree(self.temp_dir)

            if not args[0].quiet:
                print_col("Program exited with errors!", RED, self.idx)

        # Removing temporary directory, if any
        if os.path.exists(self.temp_dir):
            shutil.rmtree(self.temp_dir)


def merger(ranges):
    """
    Generator that merges ranges in a list of tuples. For example,
    if ranges is [(1, 234), (235, 456), (560, 607), (607,789)]
    this generator will yield [(1, 456), (560, 789)]
    """
    previous = 0
    last_start = 0
    for st, en in ranges:
        if not previous:
            last_start = st
            previous = en
        elif st - 1 == previous:
            previous = en
        else:
            yield last_start, previous
            previous = en
            last_start = st

    yield last_start, en


def has_colours(stream):
    if not hasattr(stream, "isatty"):
        return False
    if not stream.isatty():
        return False # auto color only on TTYs
    try:
        import curses
        curses.setupterm()
        return curses.tigetnum("colors") > 2
    except:
        # guess false in case of error
        return False

# Support for terminal colors
BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)
has_colours = has_colours(sys.stdout)


def print_col(text, color, i=0, quiet=False):
    if not quiet:
        p = ["TriSeq", "OrthoMCl Pipeline", "TriStats", "TriOrtho"]
        suf = {GREEN: "[%s] " % p[i], YELLOW: "[%s-Warning] " % p[i],
               RED: "[%s-Error] " % p[i]}
        if has_colours:
            seq = "\x1b[1;%dm" % (30 + color) + suf[color] + "\x1b[0m" + text
            print(seq)
        else:
            print(text)

    if color is RED:
        raise SystemExit


class Base(object):

    def autofinder(self, reference_file):
        """ Autodetect the type of file to be parsed. Based on headers """

        file_handle = open(reference_file, "r")
        # Set to True when the format has been detected
        format_found = False

        # If input file is not a simple text file, which means it"s invalid,
        # handle this exception
        try:
            header = file_handle.readline()
        except UnicodeDecodeError:
            return InputError("Invalid input file.")

        try:
            # Skips first empty lines, if any
            while header.startswith("\n"):
                header = next(file_handle)

            # Recognition of NEXUS files is based on the existence of the
            # string "#NEXUS" in the first non-empty line
            if header.upper().strip().startswith("#NEXUS"):
                autofind = "nexus"
                format_found = True
                while True:
                        line = next(file_handle)
                        if line.strip().lower() == "matrix":
                            next_line = next(file_handle)
                            while next_line.startswith("\n"):
                                next_line = next(file_handle)
                            sequence = "".join(next_line.split()[1:]).strip()
                            break

            # Recognition of Stockhold files is based on the existence of the
            # string "# stockholm" in the first non-empty line
            # (case insensitive)
            elif header.upper().strip().startswith("# STOCKHOLM") or \
                    header.upper().strip().startswith("#STOCKHOLM"):
                autofind = "stockholm"
                format_found = True
                while True:
                    line = file_handle.readline()
                    if not line.startswith("#") and line.strip() != "":
                        sequence = line.split()[1]
                        break

            # Recognition of FASTA or .loci files is based on the existence
            # of a ">" character as the first character of a non-empty line
            elif header.strip().startswith(">"):
                next_line = next(file_handle)
                if next_line.strip().startswith(">"):
                    autofind = "loci"
                    format_found = True
                    sequence = header.split()[-1].strip()
                else:
                    autofind = "fasta"
                    format_found = True
                    sequence = next_line.strip()
                    for line in file_handle:
                        if line.strip() != "" and line.strip()[0] != ">":
                            sequence += line.strip()
                        elif line.strip() != "" and line.strip()[0] == ">":
                            break

            # Recognition of Phylip files is based on the existence of two
            # integers separated by whitespace on the first non-empy line
            elif len(header.strip().split()) == 2 and header.strip().split()[0]\
                    .isdigit() and header.strip().split()[1].isdigit():

                autofind = "phylip"
                format_found = True
                sequence = "".join(file_handle.readline().split()[1:]).strip()

            # Recognition of ipyrad loci file, which does not start with a ">"
            # character
            if not format_found:
                while not header.startswith("//"):
                    try:
                        header = next(file_handle)
                    except StopIteration:
                        break
                    if header.startswith("//"):
                        if header.count("|") == 2:
                            autofind = "loci"
                            format_found = True
                            sequence = next(file_handle).split()[1]

            # Check if there is any sequence. If not, the alignment file has no
            # sequence
            if not format_found:
                return InputError("Unknown input file format.")
            if sequence.replace("-", "") == "":
                return EmptyAlignment("Alignment is empty")

            # Guessing the genetic code
            code = self.guess_code(sequence)

        except StopIteration:
            return EmptyAlignment("Alignment is empty")

        return autofind, code

    def get_loci_taxa(self, loci_file):
        """
        Gets a taxa list from a loci file. This is required prior to parsing
        the alignment in order to correctly add missing data when certain
        taxa are not present in a locus
        :param loci_file: string, path to loci file
        """

        file_handle = open(loci_file)
        taxa_list = []

        for line in file_handle:
            if not line.strip().startswith("//") and line.strip() != "":
                taxon = line.strip().split()[0].lstrip(">")
                if taxon not in taxa_list:
                    taxa_list.append(taxon)

        return taxa_list

    def guess_code(self, sequence):
        """ Function that guesses the code of the molecular sequences (i.e.,
        DNA or Protein) based on the first sequence of a reference file """

        # Removes gaps from the sequence so that the frequencies are not biased
        sequence = sequence.upper().replace("-", "")

        dna_count = sequence.count("A") + sequence.count("T") + \
                    sequence.count("G") + sequence.count("C") + \
                    sequence.count("N")
        dna_proportion = float(dna_count) / float(len(sequence))
        if dna_proportion > 0.9:  # The 0.9 cut-off has been effective so far
            code = ("DNA", "n")
        else:
            code = ("Protein", "x")
        return code

    def rm_illegal(self, string):
        """ Function that removes illegal characters from taxa names """

        # Additional illegal characters are added here
        illegal_chars = [":", ",", ")", "(", ";", "[", "]", """, """]

        clean_name = "".join([char for char in string if char not in
                              illegal_chars])

        return clean_name

    def duplicate_taxa(self, taxa_list):
        """ Function that identifies duplicated taxa """
        import collections
        duplicated_taxa = [x for x, y in collections.Counter(taxa_list).items()
                           if y > 1]
        return duplicated_taxa

    def check_sizes(self, sequence_data, current_file):
        """ This will make two sanity checks of the alignment contained in
        the alignment_dic object: First, it will check if none of the
        sequences is empty; If True, it will raise an error informing which
        taxa have empty sequences. If False, this will also test whether all
        sequences are of the same size and, if not, which are different
        :param sequence_data: tuple, containing taxa and sequence data.
        (taxon, sequence)
        :param current_file: string, name/path of the current input file
        """

        # Checking for taxa with empty sequences
        empty_taxa = []
        seq_list = []
        for _, taxa, seq in sequence_data:

            if seq == "":
                empty_taxa.append(taxa)
            else:
                seq_list.append(len(seq))

        if empty_taxa is []:

            print("\nInputError: The following taxa contain empty sequences "
                  "in the file %s: %s\nPlease verify and re-run the program. "
                  "Exiting...\n" % (current_file, " ".join(empty_taxa)))
            raise SystemExit

        # Checking sequence lengths
        if len(set(seq_list)) > 1:
            return False
        else:
            return True

    def read_basic_csv(self, file_handle):
        """ This will parse a simples csv file with only one column and one
        or more lines. It returns a list containing the contents of each line
        as an element. This can be used by any class/function of the process
        submodule granted that a previous check for the presence of the file
        is made """

        storage = []

        for line in file_handle:

            storage.append(line.strip())

        return storage


class Progression(object):

    def record(self, name, obj_size, window_size=50):

        self.name = name
        self.size = obj_size
        self.width = window_size

    def progress_bar(self, position):
        """ this function requires the record method to be previously defined,
        as it will need its attributes. It will print a progress bar with a
        specified weight according to the position on the current data
        structure """

        # If there is a previous message in the output, erase it
        try:
            sys.stdout.write("\r" + " " * len(self.msg))
        except AttributeError:
            pass

        # The progress bar
        position_proportion = int((position / self.size) * self.width)

        msg = "\r%s [%s%s] %s%%" % (self.name, "#" * position_proportion,
                                    "-" * (self.width - position_proportion),
                                    int((position_proportion / self.width) *
                                        100))

        # Erase the last message
        if int((position_proportion / self.width) * 100) == 100:
            sys.stdout.write("\r" + " " * len(msg))

    def write(self, msg):
        """ This will simply write a provided string to the terminal """

        self.msg = msg


__author__ = "Diogo N. Silva"
